@@ -3,15 +3,12 @@
 """
 Utils
 =====
-Utility functions required for registering all defined scientific colormaps.
+Utility functions for registering and manipulating colormaps in various ways.
 
 """
 
 
 # %% IMPORTS
-# Future imports
-from __future__ import absolute_import, division, print_function
-
 # Built-in imports
 from collections import OrderedDict as odict
 from glob import glob
@@ -28,14 +25,13 @@ from matplotlib.legend_handler import HandlerBase
 from matplotlib.image import AxesImage
 import matplotlib.pyplot as plt
 import numpy as np
-from six import string_types
 
 # CMasher imports
 from cmasher import cm as cmrcm
 
 # All declaration
-__all__ = ['create_cmap_overview', 'get_bibtex', 'get_cmap_type',
-           'get_sub_cmap', 'import_cmaps', 'register_cmap',
+__all__ = ['create_cmap_mod', 'create_cmap_overview', 'get_bibtex',
+           'get_cmap_type', 'get_sub_cmap', 'import_cmaps', 'register_cmap',
            'set_cmap_legend_entry', 'take_cmap_colors']
 
 
@@ -149,8 +145,119 @@ def _get_cmap_lightness_rank(cmap):
 
 
 # %% FUNCTIONS
+# This function creates a standalone module of a CMasher colormap
+def create_cmap_mod(cmap, *, save_dir='.'):
+    """
+    Creates a standalone Python module of the provided *CMasher* `cmap` and
+    saves it in the given `save_dir` as '<`cmap`>.py'.
+
+    A standalone colormap module can be used to quickly share a colormap with
+    someone without adding the *CMasher* dependency.
+    Importing the created module allows the colormap to be used in the same way
+    as usual through *MPL* (including the 'cmr.' prefix).
+
+    Parameters
+    ----------
+    cmap : str
+        The name of the *CMasher* colormap a standalone Python module must be
+        made for. An added 'cmr.' prefix will be ignored.
+
+    Optional
+    --------
+    save_dir : str. Default: '.'
+        The path to the directory where the module must be saved.
+        By default, the current directory is used.
+
+    Returns
+    -------
+    cmap_path : str
+        The path to the Python file containing the colormap module.
+
+    Example
+    -------
+    Creating a standalone Python module of the 'rainforest' colormap::
+
+        >>> create_cmap_mod('rainforest')
+
+    One can now import the 'rainforest' colormap in any script by moving the
+    created 'rainforest.py' file to the proper working directory and importing
+    it with ``import rainforest``.
+
+    Note
+    ----
+    Unlike other *CMasher* utility functions, `cmap` solely accepts names of
+    colormaps that are registered in *CMasher* (:mod:`cmasher.cm`).
+
+    """
+
+    # Get absolute value to provided save_dir
+    save_dir = path.abspath(save_dir)
+
+    # Remove any 'cmr.' prefix from provided cmap
+    name = cmap.replace('cmr.', '')
+
+    # Obtain the CMasher colormap associated with the provided cmap
+    cmap = cmrcm.cmap_d.get(name, None)
+
+    # If cmap is None, raise error
+    if cmap is None:
+        raise ValueError("Input argument 'cmap' is not a valid CMasher "
+                         "colormap (%r)!" % (name))
+
+    # Obtain the RGB tuples of provided cmap
+    rgb = np.array(cmap.colors)
+
+    # Convert RGB values to string
+    array_str = np.array2string(rgb, max_line_width=79, prefix='cm_data = ',
+                                separator=', ', threshold=rgb.size,
+                                formatter={'float': lambda x: "%.8f" % (x)})
+
+    # Create Python module template and add obtained RGB data to it
+    cm_py_file = dedent("""
+        # %% IMPORTS
+        # Package imports
+        from matplotlib.cm import register_cmap
+        from matplotlib.colors import ListedColormap
+
+        # All declaration
+        __all__ = ['cmap']
+
+        # Author declaration
+        __author__ = "Ellert van der Velden (@1313e)"
+
+        # Package declaration
+        __package__ = 'cmasher'
+
+
+        # %% GLOBALS AND DEFINITIONS
+        # Type of this colormap
+        cm_type = '{0}'
+
+        # RGB-values of this colormap
+        cm_data = {1}
+
+        # Create ListedColormap object for this colormap
+        cmap = ListedColormap(cm_data, name='cmr.{2}', N=len(cm_data))
+        cmap_r = cmap.reversed()
+
+        # Register (reversed) cmap in MPL
+        register_cmap(cmap=cmap)
+        register_cmap(cmap=cmap_r)
+        """).format(get_cmap_type(cmap), array_str, name)
+
+    # Obtain the path to the module
+    cmap_path = path.join(save_dir, "{0}.py".format(name))
+
+    # Create Python module
+    with open(cmap_path, 'w') as f:
+        f.write(cm_py_file[1:])
+
+    # Return cmap_path
+    return(cmap_path)
+
+
 # This function creates an overview plot of all colormaps specified
-def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
+def create_cmap_overview(cmaps=None, *, savefig=None, use_types=True,
                          sort='alphabetical', plot_profile=False,
                          title="Colormap Overview"):
     """
@@ -177,8 +284,8 @@ def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
         overview.
         If 'alphabetical', the colormaps are sorted alphabetically on their
         name.
-        If 'lightness', the colormaps are sorted on their starting lightness
-        and their lightness range.
+        If 'lightness', the colormaps are sorted based on their lightness
+        profile.
         If function, a function definition that takes a
         :obj:`~matplotlib.colors.Colormap` object and returns the sorted
         position of that colormap.
@@ -189,9 +296,9 @@ def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
         its gray-scale version and `plot_profile` is used for setting the alpha
         (opacity) value.
         If `plot_profile` is *True*, it will be set to `0.25`.
-    title : str or False. Default: "Colormap Overview"
+    title : str or None. Default: "Colormap Overview"
         String to be used as the title of the colormap overview.
-        If empty or *False*, no title will be used.
+        If empty or *None*, no title will be used.
 
     Notes
     -----
@@ -217,7 +324,7 @@ def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
         cmaps = cmrcm.cmap_d.values()
 
     # If sort is a string, obtain proper function
-    if isinstance(sort, string_types):
+    if isinstance(sort, str):
         # Convert sort to lowercase
         sort = sort.lower()
 
@@ -249,7 +356,7 @@ def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
 
             # Loop over all cmaps and remove reversed versions
             for cmap in cmaps:
-                if isinstance(cmap, string_types):
+                if isinstance(cmap, str):
                     cmaps_dict[cm_type].append(mplcm.get_cmap(cmap))
                 else:
                     cmaps_dict[cm_type].append(cmap)
@@ -266,14 +373,14 @@ def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
             # Loop over all cmaps and remove reversed versions
             for cmap in cmaps:
                 cm_type = get_cmap_type(cmap)
-                if isinstance(cmap, string_types):
+                if isinstance(cmap, str):
                     cmaps_dict[cm_type].append(mplcm.get_cmap(cmap))
                 else:
                     cmaps_dict[cm_type].append(cmap)
         else:
             # Loop over all cmaps and remove reversed versions
             for cmap in cmaps:
-                if isinstance(cmap, string_types):
+                if isinstance(cmap, str):
                     cmaps_list.append(mplcm.get_cmap(cmap))
                 else:
                     cmaps_list.append(cmap)
@@ -321,8 +428,10 @@ def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
 
     # Create figure instance
     height = 0.4*(len(cmaps_list)+bool(title))
-    fig, axes = plt.subplots(figsize=(6.4, height), nrows=len(cmaps_list),
-                             ncols=2)
+    fig, axs = plt.subplots(figsize=(6.4, height), nrows=len(cmaps_list),
+                            ncols=2)
+
+    # Add title if requested and adjust subplot positioning
     if title:
         fig.subplots_adjust(top=(1-0.288/height), bottom=0.048/height,
                             left=0.2, right=0.99, wspace=0.05)
@@ -331,23 +440,23 @@ def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
         fig.subplots_adjust(top=(1-0.048/height), bottom=0.048/height,
                             left=0.2, right=0.99, wspace=0.05)
 
-    # If cmaps_list only has a single element, make sure axes is a list
+    # If cmaps_list only has a single element, make sure axs is a list
     if(len(cmaps_list) == 1):
-        axes = [axes]
+        axs = [axs]
 
     # Set the current cm_type to None
     cm_type = None
 
     # Loop over all cmaps defined in cmaps list
-    for ax, cmap in zip(axes, cmaps_list):
+    for (ax0, ax1), cmap in zip(axs, cmaps_list):
         # Turn axes off
-        ax[0].set_axis_off()
-        ax[1].set_axis_off()
+        ax0.set_axis_off()
+        ax1.set_axis_off()
 
         # If cmap is a string, it defines a cm_type
-        if isinstance(cmap, string_types):
+        if isinstance(cmap, str):
             # Write the cm_type as text in the correct position
-            fig.text(0.595, ax[0].get_position().bounds[1], cmap,
+            fig.text(0.595, ax0.get_position().bounds[1], cmap,
                      va='bottom', ha='center', fontsize=14)
 
             # Save what the current cm_type is
@@ -372,7 +481,7 @@ def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
             rgb_L = cmrcm.neutral(L)[:, :3]
 
             # Add colormap subplot
-            ax[0].imshow(rgb[np.newaxis, ...], aspect='auto')
+            ax0.imshow(rgb[np.newaxis, ...], aspect='auto')
 
             # Check if the lightness profile was requested
             if plot_profile and (cm_type != 'qualitative'):
@@ -416,13 +525,13 @@ def create_cmap_overview(cmaps=None, savefig=None, use_types=True,
                 lc.set_array(s_colors)
 
                 # Add line-collection to this subplot
-                ax[1].add_collection(lc)
+                ax1.add_collection(lc)
 
             # Add gray-scale colormap subplot
-            ax[1].imshow(rgb_L[np.newaxis, ...], aspect='auto')
+            ax1.imshow(rgb_L[np.newaxis, ...], aspect='auto')
 
             # Plot the name of the colormap as text
-            pos = list(ax[0].get_position().bounds)
+            pos = list(ax0.get_position().bounds)
             x_text = pos[0]-0.01
             y_text = pos[1]+pos[3]/2
             fig.text(x_text, y_text, cmap.name, va='center', ha='right',
@@ -599,7 +708,7 @@ def get_sub_cmap(cmap, start, stop):
     One can check the number of colors in a colormap with
     :attr:`matplotlib.colors.Colormap.N`.
 
-    Any colormaps created using this function is not registered in either
+    Any colormaps created using this function are not registered in either
     *CMasher* or *MPL*.
 
     """
@@ -607,20 +716,11 @@ def get_sub_cmap(cmap, start, stop):
     # Obtain the colormap
     cmap = mplcm.get_cmap(cmap)
 
-    # Check if provided start and stop are valid
-    if not ((0 <= start <= 1) and (0 <= stop <= 1)):
-        raise ValueError("Input arguments 'start' and 'stop' do not contain "
-                         "normalized values!")
-
-    # Convert start and stop to their integer indices
-    start = int(np.floor(start*cmap.N))
-    stop = int(np.ceil(stop*cmap.N))
-
-    # Obtain all colors that are required for the new colormap
-    colors = cmap(np.arange(start, stop))[:, :3].tolist()
+    # Obtain colors
+    colors = take_cmap_colors(cmap, None, cmap_range=(start, stop))
 
     # Create new colormap
-    sub_cmap = LC(colors, cmap.name+'_sub', N=stop-start)
+    sub_cmap = LC(colors, cmap.name+'_sub', N=len(colors))
 
     # Return sub_cmap
     return(sub_cmap)
@@ -717,7 +817,7 @@ def import_cmaps(cmap_path):
                 try:
                     import viscm
                 # If that fails, raise error
-                except ImportError:
+                except ImportError:  # pragma: no cover
                     raise ImportError("The 'viscm' package is required to read"
                                       " '.jscm' files!")
                 # If that succeeds, load RGB values from source file
@@ -863,7 +963,7 @@ def set_cmap_legend_entry(artist, label):
 
 
 # Function to take N equally spaced colors from a colormap
-def take_cmap_colors(cmap, N, cmap_range=(0, 1), return_hex=False):
+def take_cmap_colors(cmap, N, *, cmap_range=(0, 1), return_fmt='float'):
     """
     Takes `N` equally spaced colors from the provided colormap `cmap` and
     returns them.
@@ -875,8 +975,9 @@ def take_cmap_colors(cmap, N, cmap_range=(0, 1), return_hex=False):
     cmap : str or :obj:`~matplotlib.colors.Colormap` object
         The registered name of the colormap in :mod:`matplotlib.cm` or its
         corresponding :obj:`~matplotlib.colors.Colormap` object.
-    N : int
+    N : int or None
         The number of colors to take from the provided `cmap`.
+        If *None*, take all colors in `cmap` within the provided `cmap_range`.
 
     Optional
     --------
@@ -884,18 +985,20 @@ def take_cmap_colors(cmap, N, cmap_range=(0, 1), return_hex=False):
         The normalized value range in the colormap from which colors should be
         taken.
         By default, colors are taken from the entire colormap.
-    return_hex : bool. Default: False
-        If *True*, the colors are returned using their hexadecimal string
+    return_fmt : {'float'/'norm'; 'str'/'hex'; 'int'/'8bit'}. Default: 'float'
+        The format of the requested colors.
+        If 'float'/'norm', the colors are returned as normalized RGB tuples.
+        If 'str'/'hex', the colors are returned using their hexadecimal string
         representations.
-        If *False*, they are instead returned as normalized RGB tuples.
+        If 'int'/'8bit', the colors are returned as 8-bit RGB tuples.
 
     Returns
     -------
     colors : list of tuple, str
         The colors that were taken from the provided `cmap`.
 
-    Example
-    -------
+    Examples
+    --------
     Taking five equally spaced colors from the 'rainforest' colormap::
 
         >>> take_cmap_colors('cmr.rainforest', 5)
@@ -905,15 +1008,25 @@ def take_cmap_colors(cmap, N, cmap_range=(0, 1), return_hex=False):
          (0.709615979, 0.722863985, 0.0834727592),
          (1.0, 1.0, 1.0)]
 
-    Requesting their HEX-code values instead::
+    Requesting their 8-bit RGB values instead::
 
-        >>> take_cmap_colors('cmr.rainforest', 5, return_hex=True)
-        ['#000000', '#3a2090', '#0e8474', '#b5b815', '#ffffff']
+        >>> take_cmap_colors('cmr.rainforest', 5, return_fmt='int')
+        [(0, 0, 0),
+         (58, 32, 144),
+         (14, 132, 116),
+         (181, 184, 21),
+         (255, 255, 255)]
+
+    Requesting HEX-code values instead::
+
+        >>> take_cmap_colors('cmr.rainforest', 5, return_fmt='hex')
+        ['#000000', '#3A2090', '#0E8474', '#B5B815', '#FFFFFF']
 
     Requesting colors in a specific range::
 
-        >>> take_cmap_colors('cmr.rainforest', 5, (0.2, 0.8), return_hex=True)
-        ['#3e0374', '#10528a', '#0e8474', '#5cad3c', '#d6bf4a']
+        >>> take_cmap_colors('cmr.rainforest', 5, cmap_range=(0.2, 0.8),
+                             return_fmt='hex')
+        ['#3E0374', '#10528A', '#0E8474', '#5CAD3C', '#D6BF4A']
 
     Note
     ----
@@ -924,6 +1037,9 @@ def take_cmap_colors(cmap, N, cmap_range=(0, 1), return_hex=False):
 
     """
 
+    # Convert provided fmt to lowercase
+    return_fmt = return_fmt.lower()
+
     # Obtain the colormap
     cmap = mplcm.get_cmap(cmap)
 
@@ -932,13 +1048,25 @@ def take_cmap_colors(cmap, N, cmap_range=(0, 1), return_hex=False):
         raise ValueError("Input argument 'cmap_range' does not contain "
                          "normalized values!")
 
+    # Extract and convert start and stop to their integer indices (inclusive)
+    start = int(np.floor(cmap_range[0]*cmap.N))
+    stop = int(np.ceil(cmap_range[1]*cmap.N))-1
+
     # Pick colors
-    cmap_range = np.array(cmap_range)*(cmap.N-1)
-    index = np.array(np.rint(np.linspace(*cmap_range, num=N)), dtype=int)
+    if N is None:
+        index = np.arange(start, stop+1, dtype=int)
+    else:
+        index = np.array(np.rint(np.linspace(start, stop, num=N)), dtype=int)
     colors = cmap(index)
 
-    # Convert colors to RGB tuples or hex
-    colors = list(map(to_hex if return_hex else to_rgb, colors))
+    # Convert colors to proper format
+    if return_fmt in ('float', 'norm', 'int', '8bit'):
+        colors = np.apply_along_axis(to_rgb, 1, colors)
+        if return_fmt in ('int', '8bit'):
+            colors = np.array(np.rint(colors*255), dtype=int)
+        colors = list(map(tuple, colors))
+    else:
+        colors = list(map((lambda x: to_hex(x).upper()), colors))
 
     # Return colors
     return(colors)
